@@ -304,13 +304,15 @@ copy_comment (FILE *fin, struct obstack *oout)
 /*-----------------------------------------------------------------.
 | FIN is pointing to a location (i.e., a `@').  Output to OOUT a   |
 | reference to this location. STACK_OFFSET is the number of values |
-| in the current rule so far, which says where to find `$0' with   |
+| in the current rule so far, which says where to find `@0' with   |
 | respect to the top of the stack.                                 |
 `-----------------------------------------------------------------*/
 
 static inline void
-copy_at (FILE *fin, struct obstack *oout, int stack_offset)
+copy_at (FILE *fin, struct obstack *oout,
+	 struct symbol_list *rule, int stack_offset)
 {
+  symbol_list *rp;
   int c;
 
   c = getc (fin);
@@ -321,11 +323,25 @@ copy_at (FILE *fin, struct obstack *oout, int stack_offset)
     }
   else if (isdigit (c) || c == '-')
     {
-      int n;
+      int n, i;
 
       ungetc (c, fin);
       n = read_signed_integer (fin);
 
+      rp = rule;
+      i = 0;
+
+      while (i < n)
+	{
+	  rp = rp->next;
+	  if (rp == NULL)
+	    {
+	      complain (_("invalid @ value"));
+	      return;
+	    }
+	  i++;
+	}
+
       obstack_fgrow1 (oout, "yylsp[%d]", n - stack_offset);
       locations_flag = 1;
     }
@@ -729,6 +745,7 @@ typedef union";
   const char *epilogue = "\
  yystype;\n\
 # define YYSTYPE yystype\n\
+# define YYSTYPE_IS_TRIVIAL 1\n\
 #endif\n";
 
   if (typed)
@@ -1103,7 +1120,7 @@ copy_action (symbol_list *rule, int stack_offset)
 
 	    case '@':
 	      copy_at (finput, &action_obstack,
-		       stack_offset);
+		       rule, stack_offset);
 	      break;
 
 	    case EOF:
@@ -1203,7 +1220,7 @@ copy_guard (symbol_list *rule, int stack_offset)
 	  break;
 
 	case '@':
-	  copy_at (finput, &guard_obstack, stack_offset);
+	  copy_at (finput, &guard_obstack, rule, stack_offset);
 	  break;
 
 	case EOF:
@@ -1424,7 +1441,10 @@ readgram (void)
 		  unlex (t1);
 		  symval = ssave;
 		  if (t1 == tok_colon)
-		    break;
+		    {
+		      warn (_("previous rule lacks an ending `;'"));
+		      break;
+		    }
 
 		  if (!first_rhs)	/* JF */
 		    first_rhs = symval;
@@ -1533,6 +1553,8 @@ readgram (void)
 	  /* Warn if there is no default for $$ but we need one.  */
 	  else if (!xactions && !first_rhs && lhs->type_name != 0)
 	    complain (_("empty rule for typed nonterminal, and no action"));
+	  if (t == tok_two_percents || t == tok_eof)
+	    warn (_("previous rule lacks an ending `;'"));
 	  if (t == tok_semicolon)
 	    t = lex ();
 	}
@@ -1596,12 +1618,16 @@ readgram (void)
       /* We used to use `unsigned long' as YYSTYPE on MSDOS,
          but it seems better to be consistent.
          Most programs should declare their own type anyway.  */
-      obstack_sgrow (&attrs_obstack,
-			   "#ifndef YYSTYPE\n#define YYSTYPE int\n#endif\n");
+      obstack_sgrow (&attrs_obstack, "\
+#ifndef YYSTYPE\n\
+# define YYSTYPE int\n\
+# define YYSTYPE_IS_TRIVIAL 1\n\
+#endif\n");
       if (defines_flag)
 	obstack_sgrow (&defines_obstack, "\
 # ifndef YYSTYPE\n\
 #  define YYSTYPE int\n\
+#  define YYSTYPE_IS_TRIVIAL 1\n\
 # endif\n");
     }
 
@@ -2022,6 +2048,7 @@ typedef struct yyltype\n\
 } yyltype;\n\
 \n\
 # define YYLTYPE yyltype\n\
+# define YYLTYPE_IS_TRIVIAL 1\n\
 #endif\n\
 \n");
 }
